@@ -20,13 +20,14 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <bonobo/bonobo-i18n.h>
+#include <config.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtknotebook.h>
 #include <gtk/gtkpaned.h>
 #include <libgnomedb/gnome-db-browser.h>
 #include <libgnomedb/gnome-db-list.h>
 #include <libgnomedb/gnome-db-util.h>
+#include <bonobo/bonobo-i18n.h>
 #include "gnome-db-browser-private.h"
 
 #define PARENT_TYPE GTK_TYPE_VBOX
@@ -45,7 +46,15 @@ struct _GnomeDbBrowserPrivate {
 		GtkWidget *object_list;
 		GtkWidget *object_detail;
 		gboolean data_retrieved;
-	} objects_info [6];
+		const gchar *title;
+
+		/* function pointers for each operation */
+		GtkWidget * (* new_function) (void);
+		void        (* show_function) (GtkWidget *widget,
+					       GdaConnection *cnc,
+					       const gchar *str);
+		void        (* clear_function) (GtkWidget *widget);
+	} objects_info [7];
 };
 
 static void gnome_db_browser_class_init   (GnomeDbBrowserClass *klass);
@@ -93,179 +102,154 @@ notebook_page_changed_cb (GtkNotebook *notebook,
 }
 
 static void
-table_selected_cb (GnomeDbList *list, gint row, gpointer user_data)
+object_selected_cb (GnomeDbList *list, gint row, gpointer user_data)
 {
 	GdaDataModel *model;
 	const GdaValue *value;
 	const gchar *str;
+	gint current_page;
 	GnomeDbBrowser *browser = (GnomeDbBrowser *) user_data;
 
 	g_return_if_fail (GNOME_DB_IS_BROWSER (browser));
 
+	current_page = gtk_notebook_current_page (
+		GTK_NOTEBOOK (browser->priv->object_notebook));
+
 	model = gnome_db_list_get_model (
-		GNOME_DB_LIST (browser->priv->objects_info[0].object_list));
+		GNOME_DB_LIST (browser->priv->objects_info[current_page].object_list));
 	value = gda_data_model_get_value_at (model, 0, row);
 	if (!value)
 		return;
 
 	str = gda_value_get_string ((GdaValue *) value);
 	g_signal_emit (G_OBJECT (browser), browser_signals[PROGRESS_MESSAGE],
-		       0, _("Loading table data..."));
-	gnome_db_browser_tables_show (browser->priv->objects_info[0].object_detail,
-				      browser->priv->cnc, str);
+		       0, _("Loading..."));
+	if (browser->priv->objects_info[current_page].show_function) {
+		browser->priv->objects_info[current_page].show_function (
+			browser->priv->objects_info[current_page].object_detail,
+			browser->priv->cnc,
+			str);
+	}
+
 	g_signal_emit (G_OBJECT (browser), browser_signals[PROGRESS_MESSAGE], 0, NULL);
 }
 
 static void
-table_selection_cleared_cb (GnomeDbList *list, gpointer user_data)
+object_selection_cleared_cb (GnomeDbList *list, gpointer user_data)
 {
-	GnomeDbBrowser *browser = (GnomeDbBrowser *) user_data;
-
-	g_return_if_fail (GNOME_DB_IS_BROWSER (browser));
-	gnome_db_browser_tables_clear (browser->priv->objects_info[0].object_detail);
-}
-
-static void
-view_selected_cb (GnomeDbList *list, gint row, gpointer user_data)
-{
-	GdaDataModel *model;
-	const GdaValue *value;
-	const gchar *str;
+	gint current_page;
 	GnomeDbBrowser *browser = (GnomeDbBrowser *) user_data;
 
 	g_return_if_fail (GNOME_DB_IS_BROWSER (browser));
 
-	model = gnome_db_list_get_model (
-		GNOME_DB_LIST (browser->priv->objects_info[2].object_list));
-	value = gda_data_model_get_value_at (model, 0, row);
-	if (!value)
-		return;
-
-	str = gda_value_get_string ((GdaValue *) value);
-	g_signal_emit (G_OBJECT (browser), browser_signals[PROGRESS_MESSAGE],
-		       0, _("Loading view data..."));
-	gnome_db_browser_views_show (browser->priv->objects_info[2].object_detail,
-				     browser->priv->cnc, str);
-	g_signal_emit (G_OBJECT (browser), browser_signals[PROGRESS_MESSAGE], 0, NULL);
-}
-
-static void
-view_selection_cleared_cb (GnomeDbList *list, gpointer user_data)
-{
-	GnomeDbBrowser *browser = (GnomeDbBrowser *) user_data;
-
-	g_return_if_fail (GNOME_DB_IS_BROWSER (browser));
-	gnome_db_browser_views_clear (browser->priv->objects_info[2].object_detail);
+	current_page = gtk_notebook_current_page (
+		GTK_NOTEBOOK (browser->priv->object_notebook));
+	if (browser->priv->objects_info[current_page].clear_function) {
+		browser->priv->objects_info[current_page].clear_function (
+			browser->priv->objects_info[current_page].object_detail);
+	}
 }
 
 /*
  * Private functions
  */
 
+#define POS_DATABASES  0
+#define POS_TABLES     1
+#define POS_VIEWS      2
+#define POS_TYPES      3
+#define POS_TRIGGERS   4
+#define POS_PROCEDURES 5
+#define POS_AGGREGATES 6
+
 static void
 initialize_object_widgets (GnomeDbBrowser *browser)
 {
+	gint i;
+
 	g_return_if_fail (GNOME_DB_IS_BROWSER (browser));
 
-	/* initialize tables widgets */
-	browser->priv->objects_info[0].schema = GDA_CONNECTION_SCHEMA_TABLES;
-	browser->priv->objects_info[0].data_retrieved = FALSE;
+	/* initialize function pointers */
+	browser->priv->objects_info[POS_DATABASES].schema = GDA_CONNECTION_SCHEMA_DATABASES;
+	browser->priv->objects_info[POS_DATABASES].data_retrieved = FALSE;
+	browser->priv->objects_info[POS_DATABASES].title = _("Databases");
+	browser->priv->objects_info[POS_DATABASES].new_function = NULL;
+	browser->priv->objects_info[POS_DATABASES].show_function = NULL;
+	browser->priv->objects_info[POS_DATABASES].clear_function = NULL;
 
-	browser->priv->objects_info[0].object_list = gnome_db_list_new ();
-	g_signal_connect (G_OBJECT (browser->priv->objects_info[0].object_list),
-			  "row_selected", G_CALLBACK (table_selected_cb), browser);
-	g_signal_connect (G_OBJECT (browser->priv->objects_info[0].object_list),
-			  "selection_cleared", G_CALLBACK (table_selection_cleared_cb),
-			  browser);
-	gtk_notebook_append_page (GTK_NOTEBOOK (browser->priv->object_notebook),
-				  browser->priv->objects_info[0].object_list,
-				  gtk_label_new (_("Tables")));
-	gtk_widget_show (browser->priv->objects_info[0].object_list);
+	browser->priv->objects_info[POS_TABLES].schema = GDA_CONNECTION_SCHEMA_TABLES;
+	browser->priv->objects_info[POS_TABLES].data_retrieved = FALSE;
+	browser->priv->objects_info[POS_TABLES].title = _("Tables");
+	browser->priv->objects_info[POS_TABLES].new_function = gnome_db_browser_tables_new;
+	browser->priv->objects_info[POS_TABLES].show_function = gnome_db_browser_tables_show;
+	browser->priv->objects_info[POS_TABLES].clear_function = gnome_db_browser_tables_clear;
 
-	browser->priv->objects_info[0].object_detail = gnome_db_browser_tables_new ();
-	gtk_notebook_append_page (GTK_NOTEBOOK (browser->priv->detail_notebook),
-				  browser->priv->objects_info[0].object_detail,
-				  gtk_label_new (_("Tables")));
+	browser->priv->objects_info[POS_TYPES].schema = GDA_CONNECTION_SCHEMA_TYPES;
+	browser->priv->objects_info[POS_TYPES].data_retrieved = FALSE;
+	browser->priv->objects_info[POS_TYPES].title = _("Types");
+	browser->priv->objects_info[POS_TYPES].new_function = gnome_db_browser_types_new;
+	browser->priv->objects_info[POS_TYPES].show_function = NULL;
+	browser->priv->objects_info[POS_TYPES].clear_function = NULL;
 
-	/* initialize types widgets */
-	browser->priv->objects_info[1].schema = GDA_CONNECTION_SCHEMA_TYPES;
-	browser->priv->objects_info[1].data_retrieved = FALSE;
+	browser->priv->objects_info[POS_VIEWS].schema = GDA_CONNECTION_SCHEMA_VIEWS;
+	browser->priv->objects_info[POS_VIEWS].data_retrieved = FALSE;
+	browser->priv->objects_info[POS_VIEWS].title = _("Views");
+	browser->priv->objects_info[POS_VIEWS].new_function = gnome_db_browser_views_new;
+	browser->priv->objects_info[POS_VIEWS].show_function = gnome_db_browser_views_show;
+	browser->priv->objects_info[POS_VIEWS].clear_function = gnome_db_browser_views_clear;
 
-	browser->priv->objects_info[1].object_list = gnome_db_list_new ();
-	gtk_notebook_append_page (GTK_NOTEBOOK (browser->priv->object_notebook),
-				  browser->priv->objects_info[1].object_list,
-				  gtk_label_new (_("Types")));
-	gtk_widget_show (browser->priv->objects_info[1].object_list);
+	browser->priv->objects_info[POS_PROCEDURES].schema = GDA_CONNECTION_SCHEMA_PROCEDURES;
+	browser->priv->objects_info[POS_PROCEDURES].data_retrieved = FALSE;
+	browser->priv->objects_info[POS_PROCEDURES].title = _("Procedures");
+	browser->priv->objects_info[POS_PROCEDURES].new_function = gnome_db_browser_procedures_new;
+	browser->priv->objects_info[POS_PROCEDURES].show_function = NULL;
+	browser->priv->objects_info[POS_PROCEDURES].clear_function = NULL;
 
-	browser->priv->objects_info[1].object_detail = gnome_db_browser_types_new ();
-	gtk_notebook_append_page (GTK_NOTEBOOK (browser->priv->detail_notebook),
-				  browser->priv->objects_info[1].object_detail,
-				  gtk_label_new (_("Types")));
+	browser->priv->objects_info[POS_AGGREGATES].schema = GDA_CONNECTION_SCHEMA_AGGREGATES;
+	browser->priv->objects_info[POS_AGGREGATES].data_retrieved = FALSE;
+	browser->priv->objects_info[POS_AGGREGATES].title = _("Aggregates");
+	browser->priv->objects_info[POS_AGGREGATES].new_function = NULL;
+	browser->priv->objects_info[POS_AGGREGATES].show_function = NULL;
+	browser->priv->objects_info[POS_AGGREGATES].clear_function = NULL;
 
-	/* initialize views widgets */
-	browser->priv->objects_info[2].schema = GDA_CONNECTION_SCHEMA_VIEWS;
-	browser->priv->objects_info[2].data_retrieved = FALSE;
+	browser->priv->objects_info[POS_TRIGGERS].schema = GDA_CONNECTION_SCHEMA_TRIGGERS;
+	browser->priv->objects_info[POS_TRIGGERS].data_retrieved = FALSE;
+	browser->priv->objects_info[POS_TRIGGERS].title = _("Triggers");
+	browser->priv->objects_info[POS_TRIGGERS].new_function = NULL;
+	browser->priv->objects_info[POS_TRIGGERS].show_function = NULL;
+	browser->priv->objects_info[POS_TRIGGERS].clear_function = NULL;
 
-	browser->priv->objects_info[2].object_list = gnome_db_list_new ();
-	g_signal_connect (G_OBJECT (browser->priv->objects_info[2].object_list),
-			  "row_selected", G_CALLBACK (view_selected_cb), browser);
-	g_signal_connect (G_OBJECT (browser->priv->objects_info[2].object_list),
-			  "selection_cleared", G_CALLBACK (view_selection_cleared_cb),
-			  browser);
-	gtk_notebook_append_page (GTK_NOTEBOOK (browser->priv->object_notebook),
-				  browser->priv->objects_info[2].object_list,
-				  gtk_label_new (_("Views")));
-	gtk_widget_show (browser->priv->objects_info[2].object_list);
+	/* create widgets */
+	for (i = 0;
+	     i < sizeof (browser->priv->objects_info) / sizeof (browser->priv->objects_info[0]);
+	     i++) {
+		GtkWidget *detail;
 
-	browser->priv->objects_info[2].object_detail = gnome_db_browser_views_new ();
-	gtk_notebook_append_page (GTK_NOTEBOOK (browser->priv->detail_notebook),
-				  browser->priv->objects_info[2].object_detail,
-				  gtk_label_new (_("Views")));
+		browser->priv->objects_info[i].object_list = gnome_db_list_new ();
+		g_signal_connect (G_OBJECT (browser->priv->objects_info[i].object_list),
+				  "row_selected", G_CALLBACK (object_selected_cb),
+				  browser);
+		g_signal_connect (G_OBJECT (browser->priv->objects_info[i].object_list),
+				  "selection_cleared",
+				  G_CALLBACK (object_selection_cleared_cb), browser);
+		gtk_widget_show (browser->priv->objects_info[i].object_list);
+		gtk_notebook_append_page (
+			GTK_NOTEBOOK (browser->priv->object_notebook),
+			browser->priv->objects_info[i].object_list,
+			gtk_label_new (browser->priv->objects_info[i].title));
 
-	/* initialize procedures widgets */
-	browser->priv->objects_info[3].schema = GDA_CONNECTION_SCHEMA_PROCEDURES;
-	browser->priv->objects_info[3].data_retrieved = FALSE;
+		if (browser->priv->objects_info[i].new_function != NULL) {
+			detail = browser->priv->objects_info[i].new_function ();
+			gtk_widget_show (detail);
+		}
+		else {
+			detail = gnome_db_new_label_widget ("TODO");
+		}
 
-	browser->priv->objects_info[3].object_list = gnome_db_list_new ();
-	gtk_notebook_append_page (GTK_NOTEBOOK (browser->priv->object_notebook),
-				  browser->priv->objects_info[3].object_list,
-				  gtk_label_new (_("Procedures")));
-	gtk_widget_show (browser->priv->objects_info[3].object_list);
-
-	browser->priv->objects_info[3].object_detail = gnome_db_browser_procedures_new ();
-	gtk_notebook_append_page (GTK_NOTEBOOK (browser->priv->detail_notebook),
-				  browser->priv->objects_info[3].object_detail,
-				  gtk_label_new (_("Procedures")));
-
-	/* initialize aggregate widgets */
-	browser->priv->objects_info[4].schema = GDA_CONNECTION_SCHEMA_AGGREGATES;
-	browser->priv->objects_info[4].data_retrieved = FALSE;
-
-	browser->priv->objects_info[4].object_list = gnome_db_list_new ();
-	gtk_notebook_append_page (GTK_NOTEBOOK (browser->priv->object_notebook),
-				  browser->priv->objects_info[4].object_list,
-				  gtk_label_new (_("Aggregates")));
-	gtk_widget_show (browser->priv->objects_info[4].object_list);
-
-	browser->priv->objects_info[4].object_detail = gnome_db_new_label_widget ("TODO");
-	gtk_notebook_append_page (GTK_NOTEBOOK (browser->priv->detail_notebook),
-	                          browser->priv->objects_info[4].object_detail,
-				  gtk_label_new (_("Aggregates")));
-
-	/* initialize trigger widgets */
-	browser->priv->objects_info[5].schema = GDA_CONNECTION_SCHEMA_TRIGGERS;
-	browser->priv->objects_info[5].data_retrieved = FALSE;
-
-	browser->priv->objects_info[5].object_list = gnome_db_list_new ();
-	gtk_notebook_append_page (GTK_NOTEBOOK (browser->priv->object_notebook),
-				  browser->priv->objects_info[5].object_list,
-				  gtk_label_new (_("Triggers")));
-	gtk_widget_show (browser->priv->objects_info[5].object_list);
-
-	browser->priv->objects_info[5].object_detail = gnome_db_new_label_widget ("TODO");
-	gtk_notebook_append_page (GTK_NOTEBOOK (browser->priv->detail_notebook),
-	                          browser->priv->objects_info[5].object_detail,
-				  gtk_label_new (_("Triggers")));
+		gtk_notebook_append_page (GTK_NOTEBOOK (browser->priv->detail_notebook),
+					  detail, gtk_label_new ("TAB"));
+		browser->priv->objects_info[i].object_detail = detail;
+	}
 }
 
 /*
@@ -481,14 +465,9 @@ gnome_db_browser_set_connection (GnomeDbBrowser *browser, GdaConnection *cnc)
 		     NULL, 0);
 		browser->priv->objects_info[n].data_retrieved = FALSE;
 
-		switch (browser->priv->objects_info[n].schema) {
-		case GDA_CONNECTION_SCHEMA_TABLES :
-			gnome_db_browser_tables_clear (browser->priv->objects_info[n].object_detail);
-			break;
-		case GDA_CONNECTION_SCHEMA_VIEWS :
-			gnome_db_browser_views_clear (browser->priv->objects_info[n].object_detail);
-			break;
-		default :
+		if (browser->priv->objects_info[n].clear_function != NULL) {
+			browser->priv->objects_info[n].clear_function (
+				browser->priv->objects_info[n].object_detail);
 		}
 	}
 
@@ -532,6 +511,7 @@ gnome_db_browser_refresh (GnomeDbBrowser *browser)
 
 	g_signal_emit (G_OBJECT (browser), browser_signals[PROGRESS_MESSAGE],
 		       0, _("Refreshing data..."));
+
 	current = gtk_notebook_get_current_page (
 		GTK_NOTEBOOK (browser->priv->object_notebook));
 
@@ -547,14 +527,9 @@ gnome_db_browser_refresh (GnomeDbBrowser *browser)
 	g_object_unref (G_OBJECT (recset));
 
 	/* clear the detail widget */
-	switch (browser->priv->objects_info[current].schema) {
-	case GDA_CONNECTION_SCHEMA_TABLES :
-		gnome_db_browser_tables_clear (browser->priv->objects_info[current].object_detail);
-		break;
-	case GDA_CONNECTION_SCHEMA_VIEWS :
-		gnome_db_browser_views_clear (browser->priv->objects_info[current].object_detail);
-		break;
-	default :
+	if (browser->priv->objects_info[current].clear_function != NULL) {
+		browser->priv->objects_info[current].clear_function (
+			browser->priv->objects_info[current].object_detail);
 	}
 
 	g_signal_emit (G_OBJECT (browser), browser_signals[PROGRESS_MESSAGE], 0, NULL);
