@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <bonobo/bonobo-i18n.h>
+#include <config.h>
 #include <libgda/gda-config.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libgnomeui/gnome-druid-page-edge.h>
@@ -28,6 +28,7 @@
 #include <libgnomedb/gnome-db-dsn-config-druid.h>
 #include <libgnomedb/gnome-db-provider-selector.h>
 #include <libgnomedb/gnome-db-util.h>
+#include <bonobo/bonobo-i18n.h>
 
 #define PARENT_TYPE GNOME_TYPE_DRUID
 
@@ -267,7 +268,7 @@ gnome_db_dsn_config_druid_init (GnomeDbDsnConfigDruid *druid,
 	druid->priv->provider_params = g_hash_table_new (g_str_hash, g_str_equal);
 
 	/* load icons */
-	druid->priv->logo = gdk_pixbuf_new_from_file (LIBGNOMEDB_PIXMAPSDIR "/gnome-db.png", NULL);
+	druid->priv->logo = gdk_pixbuf_new_from_file (LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
 
 	/* set up widgets */
 	gnome_druid_set_show_help (GNOME_DRUID (druid), FALSE);
