@@ -20,10 +20,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <bonobo/bonobo-i18n.h>
+#include <config.h>
 #include <gtk/gtktable.h>
 #include <libgnomedb/gnome-db-dsn-config.h>
+#include <libgnomedb/gnome-db-provider-selector.h>
 #include <libgnomedb/gnome-db-util.h>
+#include <bonobo/bonobo-i18n.h>
 
 #define PARENT_TYPE GTK_TYPE_VBOX
 
@@ -46,28 +48,6 @@ static GObjectClass *parent_class = NULL;
  * Private functions
  */
 
-static void
-fill_provider_menu (GnomeDbDsnConfig *config)
-{
-	GList *list;
-	GList *l;
-
-	list = gda_config_get_provider_list ();
-
-	for (l = list; l != NULL; l = l->next) {
-		GdaProviderInfo *prov_info;
-
-		prov_info = (GdaProviderInfo *) l->data;
-		if (prov_info) {
-			gnome_db_option_menu_add_item (
-				GTK_OPTION_MENU (config->priv->wprovider),
-				prov_info->id);
-		}
-	}
-
-	gda_config_free_provider_list (list);
-}
-
 /*
  * GnomeDbDsnConfig class implementation
  */
@@ -105,8 +85,8 @@ gnome_db_dsn_config_init (GnomeDbDsnConfig *config, GnomeDbDsnConfigClass *klass
 
 	label = gnome_db_new_label_widget (_("Provider"));
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 2, 2);
-	config->priv->wprovider = gnome_db_new_option_menu_widget ();
-	fill_provider_menu (config);
+	config->priv->wprovider = gnome_db_provider_selector_new ();
+	gtk_widget_show (config->priv->wprovider);
 	gtk_table_attach (GTK_TABLE (table), config->priv->wprovider, 1, 2, 1, 2,
 			  GTK_FILL, GTK_FILL, 2, 2);
 
@@ -212,7 +192,8 @@ gnome_db_dsn_config_set_info (GnomeDbDsnConfig *config, const GdaDataSourceInfo
 	g_return_if_fail (dsn_info != NULL);
 
 	gtk_entry_set_text (GTK_ENTRY (config->priv->wname), dsn_info->name);
-	//wprovider
+	gnome_db_option_menu_set_selection (GTK_OPTION_MENU (config->priv->wprovider),
+					    dsn_info->provider);
 	gtk_entry_set_text (GTK_ENTRY (config->priv->wstring), dsn_info->cnc_string);
 	gtk_entry_set_text (GTK_ENTRY (config->priv->wdesc), dsn_info->description);
 	gtk_entry_set_text (GTK_ENTRY (config->priv->wuser), dsn_info->username);
