@@ -21,10 +21,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
-
 #include <libgda/gda-error.h>
 #include <libgnomedb/gnome-db-error.h>
 #include <libgnomedb/gnome-db-util.h>
@@ -33,8 +30,7 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtktextview.h>
-
-#include "gnome-db-intl.h"
+#include <bonobo/bonobo-i18n.h>
 
 #define PARENT_TYPE GTK_TYPE_VBOX
 
