@@ -20,6 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include <libgda/gda-util.h>
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkradiobutton.h>
@@ -36,7 +37,7 @@
 #include <libgnomedb/gnome-db-table-editor.h>
 #include <libgnomeui/gnome-file-entry.h>
 #include <libgnomeui/gnome-window-icon.h>
-#include "gnome-db-intl.h"
+#include <bonobo/bonobo-i18n.h>
 #include "libgnomedb-private.h"
 
 #define PARENT_TYPE GTK_TYPE_VBOX
@@ -99,7 +100,7 @@ menu_describe_cb (GtkWidget *widget, gpointer user_data)
 		GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
 		NULL);
 	gnome_window_icon_set_from_file (GTK_WINDOW (dialog),
-					 LIBGNOMEDB_PIXMAPSDIR "/gnome-db.png");
+					 LIBGNOMEDB_ICONSDIR "/gnome-db.png");
 
 	description = gnome_db_table_editor_new ();
 	gtk_widget_show (description);
@@ -133,7 +134,7 @@ menu_save_as_cb (GtkWidget *widget, gpointer user_data)
 		GTK_STOCK_OK, GTK_RESPONSE_OK,
 		NULL);
 	gnome_window_icon_set_from_file (GTK_WINDOW (dialog),
-					 LIBGNOMEDB_PIXMAPSDIR "/gnome-db.png");
+					 LIBGNOMEDB_ICONSDIR "/gnome-db.png");
 	table = gnome_db_new_table_widget (6, 2, FALSE);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), table, TRUE, TRUE, 0);
 
@@ -217,7 +218,7 @@ menu_view_detail_cb (GtkWidget *widget, gpointer user_data)
 		GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
 		NULL);
 	gnome_window_icon_set_from_file (GTK_WINDOW (dialog),
-					 LIBGNOMEDB_PIXMAPSDIR "/gnome-db.png");
+					 LIBGNOMEDB_ICONSDIR "/gnome-db.png");
 
 	form = gnome_db_form_new ();
 	gtk_widget_show (form);
